% example1
% written by Alessandro Mennuni
% purpuse:
% illustrate the algorithm in dynamic_perturbation
%{
here dynamic_perturbation is used to solve the n sectors stocastic growth 
model with exogenous labor.
c=ones(1,n)*(A.*k^.a*l.^(1-a)+(1-delta).*k-k')
1/c(i)=1/c(i)'*beta*(1+A(i)ak(i)^(a-1)*l^(1-a)-delta) for i=1,...,n
log(A(i,t+1))=rho(i)*log(A(i,t))+eps(i,t)
when delta=1
the sol is
k'=beta*a.**A.*k.^a
c=(1-beta*a)'*A.*.k^a
%}

clear all
 global m_eq n_var
%define model parameters as global
 global beta alpha rho delta T n lab
n=2; %number of sectors. User can increase it but figures at the end will 
%be harder to interpret.
ord_=2;% order of taylor expansion at the ss,
%(if ord_=1 1st order, if ord_>1 2nd order) 
%for large models the hessian is huge and matlab may run out of memory. 
%better first ordrer in that case.

nx=2*n;%numb of state vars
nA=n;%numb of shocks <=nx
ny=n+1;%numb of jumps (consumption, l1,...,ln)
n_var=2*(nx+ny);

m_eq=3*n+1;% numb of equations of the model
T=50; %time series for initial simul, if not sufficient to reach ss, 
%increase it.
t_end=40;% period of stochastic simul

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %parameters of the model
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
beta=.99;% discount factor
alpha=zeros(n,1);% capital share in cobb douglas
rho=zeros(n,1); % persistence parameter of AR1 process
delta=zeros(n,1); %depreciation rate
var_e=zeros(n,1); % variance of the shocks
for i=1:n
    alpha(i,1)=.33;
    rho(i,1)=.99;
    delta(i,1)=1;
    var_e(i,1)=.007;
end
sigm=zeros(nx,nx);% matrix of perturbation of variances of shocks to state vars
sigm_true=zeros(nx,nx);% matrix of variances of shocks to state vars
for i=1:n
    sigm(nx-nA+i,nx-nA+i)=0;
    sigm_true(nx-nA+i,nx-nA+i)=var_e(i);
end
%sigm=[0,0;0,var_e];
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%provide ss
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% [k1',..kn',A1',...An',c',l1',..ln',k1,..kn,A1,...An,c,l1,..ln];
lab=zeros(n,1)+.3;
x_=zeros(2*(2*n+1),1);
for i=1:n
    x_(i)=((1/beta-1+delta(i))/alpha(i))^(1/(alpha(i)-1))*lab(i);%k_i'
    x_(n+i)=0;%A_i'
    x_(2*n+1+i)=lab(i);%l_i'
    x_(3*n+1+i)=((1/beta-1+delta(i))/alpha(i))^(1/(alpha(i)-1))*lab(i);%k_i
    x_(3*n+1+n+i)=0;%A_i
    x_(3*n+1+2*n+1+i)=lab(i);%l_i
end
    %consumption
for i=1:n
    x_(2*n+1)=x_(2*n+1)+x_(i)^alpha(i)*lab(i)^(1-alpha(i))-delta(i)*x_(i);%c'
    x_(5*n+2)=x_(5*n+2)+x_(i)^alpha(i)*lab(i)^(1-alpha(i))-delta(i)*x_(i);%c
end
if max(abs(n_sectors_rbc(x_)))>.001
    disp('wrong ss')
end
        
%{
x1=x_(1:nx,1);% state vars tomorrow
y1=x_(nx+1,1);% jump tomoroow
x=x_(nx+2:2*nx+ny,1);% state vars 
y=x_(2*nx+ny+1,1);%jump today
%}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%det initial point x_0 in the state space
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%k,log(A)
x_0=[x_(1:nx-nA)*.2;zeros(nA,1)-.5];
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%gen time series for the stochastic process
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

eps_s=zeros(nx,T);
randn('state',3)
eps_s(nx-nA+1:nx,1:t_end)=randn(nA,t_end);
%eps_s(nx-nA+1:nx,t_end+1:T)=zeros(nA,T-t_end);% from t_end to T no shocks,
%so that the model converges to ss.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%call main code        
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[y_sim_local,x_sim_local,y_sim_lev,x_sim_lev,ff,pp,error_local,error_global] =dynamic_perturbation('n_sectors_rbc','n_sectors_rbc_adjustm_1st_ord_',sigm,sigm_true,x_,nx,nA,ny,x_0,T,t_end,eps_s,ord_);
                  
disp('end')           
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%check residuals from figures
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

figure
subplot(1,2,1)
plot(1:T,x_sim_lev(1:n,:),1:T,x_sim_local(1:n,:),'--','LineWidth',2)
xlabel('k')
 h = legend('global','local');
set(h,'Interpreter','none')
subplot(1,2,2)
plot(1:T,x_sim_lev(n+1:2*n,:),1:T, x_sim_local(n+1:2*n,:),'--','LineWidth',2) 
xlabel('A')
 h = legend('global','local');
set(h,'Interpreter','none')
  

figure
subplot(1,2,1)
plot(1:18,x_sim_lev(1:n,1:18),1:18,x_sim_local(1:n,1:18),'--','LineWidth',2)
xlabel('k')
 h = legend('global','local',2);
set(h,'Interpreter','none')
title('ZOOM ON FIRST FEW OBSERVATIONS')
subplot(1,2,2)
plot(1:18,x_sim_lev(n+1:2*n,1:18),1:18, x_sim_local(n+1:2*n,1:18),'--','LineWidth',2)
xlabel('A')
 h = legend('global','local',2);
set(h,'Interpreter','none')


figure
plot(1:T,y_sim_lev(1,:),1:T,y_sim_local(1,:),'--','LineWidth',2)
xlabel('c vs c local')
 h = legend('global','local',2);
set(h,'Interpreter','none')
title('ZOOM ON FIRST FEW OBSERVATIONS')



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%compare with analytical solution of the model
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
disp('compare local and global solution with analytical solution of the model')
gamma=1-alpha*beta;
x_true=zeros(nx,T);
y_true=zeros(ny,T);
%k(i)'=a*beta*A(i)*k(i)^a*l(i)^(1-a)
%c=(1-a*beta)*(A*k^a)*ones(n,1)

    x_true(:,1)=x_0;
for t=1:T-1
    for i=1:n
        x_true(i,t+1)=exp(x_true(n+i,t))*x_true(i,t).^alpha(i)*.3.^(1-alpha(i))*(1-gamma(i));%k(i)
        x_true(n+i,t+1)=rho(i)*x_true(n+i,t)+sigm_true(n+i,n+i)*eps_s(n+i,t+1);%A(i)
        y_true(1+i,t)=.3;%l(i)
    end
    y_true(1,t)=gamma'*(exp(x_true(n+1:2*n,t)).*x_true(1:n,t).^alpha.*.3.^(1-alpha));%c
end
y_true(1,T)=gamma'*(exp(x_true(n+1:2*n,T)).*x_true(1:n,T).^alpha.*.3.^(1-alpha));%c
y_true(2:n+1,T)=.3;

error_x_global=abs(x_sim_lev(:,1:t_end)-x_true(:,1:t_end));
error_y_global=abs(y_sim_lev(:,1:t_end)-y_true(:,1:t_end));
error_x_local =abs(x_sim_local(:,1:t_end)-x_true(:,1:t_end));
error_y_local =abs(y_sim_local(:,1:t_end)-y_true(:,1:t_end));

disp('max error |x_{global,t}-x_{true,t}|')
disp(max(max(error_x_global)))
disp('max error |x_{local,t}-x_{true,t}|')
disp(max(max(error_x_local)))
disp('max error |y_{global,t}-y_{true,t}|')
disp(max(max(error_y_global)))
disp('max error |y_{local,t}-y_{true,t}|')
disp(max(max(error_y_local)))

disp('how many times smaller is the largest error?')
max(max(error_x_local))/max(max(error_x_global))
max(max(error_y_local))/max(max(error_y_global))


 
t_end_=t_end;% to focus on the actual simulation, with shocks
figure
subplot(1,2,1)
plot(1:t_end_,x_sim_lev(1:n,1:t_end_),'.',1:t_end_,x_true(1:n,1:t_end_)','--',1:t_end_,x_sim_local(1:n,1:t_end_)','-','LineWidth',2)
xlabel('k')
 h = legend('global','true','local');
set(h,'Interpreter','none')
subplot(1,2,2)
plot(1:t_end_,x_sim_lev(n+1:2*n,1:t_end_),'.',1:t_end_, x_true(n+1:2*n,1:t_end_)','--',1:t_end_,x_sim_local(n+1:2*n,1:t_end_)','-','LineWidth',2)
xlabel('A') 
 h = legend('global','true','local');
set(h,'Interpreter','none') 

figure
subplot(1,2,1)
plot(1:t_end_,y_sim_lev(1,1:t_end_),'.',1:t_end_,y_true(1,1:t_end_)','--',1:t_end_,y_sim_local(1,1:t_end_)','-','LineWidth',2)
xlabel('c')
 h = legend('global','true','local');
set(h,'Interpreter','none')
subplot(1,2,2)
plot(1:t_end_,y_sim_lev(2,1:t_end_),1:t_end_, y_true(2,1:t_end_)','--','LineWidth',2)
xlabel('l')
 h = legend('global','true',2);
set(h,'Interpreter','none')

  
figure
subplot(1,2,1)
plot(1:t_end_,x_sim_lev(1:n,1:t_end_),'-',1:t_end_,x_true(1:n,1:t_end_)','.',1:t_end_,x_sim_local(1:n,1:t_end_)','--','LineWidth',1.5)
title('K')
 h = legend('dynamic','true','local');
set(h,'Interpreter','none')
subplot(1,2,2)
plot(1:t_end_,y_sim_lev(1,1:t_end_),'-',1:t_end_,y_true(1,1:t_end_)','.',1:t_end_,y_sim_local(1,1:t_end_)','--','LineWidth',1.5)
title('C')
 h = legend('dynamic','true','local');
set(h,'Interpreter','none')


    