%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Function: grad
% Written by Alessandro Mennuni Sept 2010
% updated on the basis of the function centgrad written by Paul Klain
% Purpose:
% gradient and hessian computation of the function "moment"=f(x1,y1,x,y)
% Input:
% moment: function defined by user
% x1 state variables at t+1
% y1 jump variables at t+1
% x state variables at t
% y jump variables at t
% hes dummy equal to 1 if hessian is needed
% output:   
% gradient and Hessian as in klein and gomme
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [f_grad,hessian]=grad(moment,x1,y1,x,y,hes)
global m_eq n_var
%eps_=10^-4;
xx=[x1;y1;x;y];
eps_ = max(norm(xx),1)*(1e-6);

f_grad=zeros(m_eq,n_var);

f_ss=feval(moment,xx);
for j=1:n_var
        eps=zeros(n_var,1);
        eps(j)=eps_;
        ff=feval(moment,xx+eps);
        %ff=moment(xxx);
        f_grad(:,j)=(ff-f_ss)/eps(j);
end

%{
for j=1:n_var
    eps=zeros(n_var,1);
    eps(j)=eps_;
    ff1=feval(moment,xx+eps);
    ff2=feval(moment,xx-eps);
    %ff=moment(xxx);
    f_grad(:,j)=(ff1-ff2)/(2*eps(j));
end
%}
if hes==1 %compute also hessian
    hessian=zeros(m_eq*n_var,n_var);  
    s1=0;%which row of the gradient is being differenciated
    for i=1:m_eq
        %compute grad of f_grad(i,;);
        f_grad_del=zeros(n_var,n_var);
        for ii=1:n_var
              s1=s1+1;
              for j=1:n_var
                eps=zeros(n_var,1);
                eps(ii)=eps_;
                eps2=eps;
                eps2(j)=eps2(j)+eps_;
                ff_del_del=feval(moment,xx+eps2);
                ff_del=feval(moment,xx+eps);
                f_grad_del(ii,j)=(ff_del_del(i)-ff_del(i))/eps_;%dfi/dxj
               hessian(s1,j)= (f_grad_del(ii,j)-f_grad(i,j))/eps_;
               %dfi^2/dxj/dxj
              end
        end
    end
else
    hessian=0;
end
        
    