% function n_sectors_rbc_adjustm_1st_ord_
% written by Alessandro Mennuni
% contains the equilibrium conditions of the RBC model with exogenous labor
% as a function of a vector xx=[x1;y] and a policy function to compute 
% y1=gy(x1), which linearized reads as y1=y_+ff*(x1-x_), 
% so fun2(xx,x_,y_,ff_,x).
%
% input;
% xx: vector equal to [x1;y]
% x_ y_: points x,y around which it has been approximated gy(x1)
% ff_: matrix for the linear approximation of gy(x1)~y_+ff*(x1-x_)
% x_now: vector of state variables.
% output;
% residuals from equilibrium conditions of the model.
% this function is called by dynamic_perturbation.
function [res] = n_sectors_rbc_adjustm_1st_ord_(xx,x_,y_,ff_,x_now)
%states
nx=size(x_);
y_pr=ff_* (xx(1:nx,1)-x_)+y_;

%from here onward is specific to each problem
global beta alpha rho delta n lab

k=x_now(1:n);A=x_now(n+1:2*n);
%xx contains x1 and y; states tomorrow and jumps today
%states tomorrow
k_1=xx(1:n);A_1=xx(n+1:2*n);

% jumps today
c=xx(2*n+1);l=xx(2*n+1+1:2*n+1+n);

c_1=y_pr(1);
l_1=y_pr(1+1:1+n);

%write equilibrium conditions as in n_sectors_rbc
%k_1=x(1);A_1=x(2);c_1=x(3); k=x(4);A=x(5);c=x(6);
res=zeros(3*n+1,1);
for is=1:n
    res(is)=c_1-beta*c*(1+alpha(is)*exp(A_1(is))*k_1(is)^(alpha(is)-1)*...
        (l_1(is))^(1-alpha(is))-delta(is));
    res(n+is)=A_1(is)-rho(is)*A(is);
    res(2*n+is)=l(is)-lab(is);
end
    res(3*n+1)=c+(k_1-k.*(1-delta)-exp(A).*k.^alpha.*l.^(1-alpha))'*ones(n,1); 
